@@ -7,19 +7,13 @@ control.py
 Typically, one would call `load_configuration`, then `setup`, and finally
 one of the `launch_*` functions.
 
-Dependencies:
-
-- `jinja2` for report rendering via HTML templates.
-- `pygments` for code highlighting in reports.
-- `importlib_resources` for resource loading.
-
 This module relies on a configuration file to find task meta-data, task
 specifications, and the submission it will evaluate. Call
 `load_configuration` to load `config.py` (or another named module) from
-he current directory.
+the current directory.
 
 See `potluck.default_config` for a configuration file template; values
-not specified in a custom config file will be pulled from this file.
+not specified in a custom config file will be pulled from that file.
 """
 
 import sys
@@ -39,6 +33,7 @@ from . import contexts
 from . import meta
 from . import snippets
 
+
 #---------#
 # Globals #
 #---------#
@@ -366,105 +361,161 @@ def test_specification(task_info, examples_dir):
 
     any_failed = False
 
-    by_user = meta.get_expectations(spec)
-    if len(by_user) == 0:
-        logging.log(
-            "No explicit expectations; only testing solution code."
-        )
-    else:
-        total = sum(len(exp) for exp in by_user.values())
-        logging.log(
-            f"Found {total} expectation(s) for {len(by_user)} example"
-            f" submission(s)."
-        )
-        failed = []
-        for username in by_user:
-            # Resolve which file we're targeting
-            user_folder = os.path.join(
-                examples_dir,
-                username
+    # Check both evaluation and validation modes
+    for mode in ["evaluation", "validation"]:
+        logging.log("Testing {}...".format(mode))
+        by_user = meta.get_expectations(spec, mode)
+        if by_user is None or len(by_user) == 0:
+            logging.log(
+                "No explicit expectations; only testing solution code."
             )
-            task_folder = os.path.join(user_folder, task_info["id"])
-            submission_target = os.path.join(
-                task_folder,
-                task_info["target"]
+        else:
+            total = sum(len(exp) for exp in by_user.values())
+            logging.log(
+                f"Found {total} expectation(s) for {len(by_user)} example"
+                f" submission(s)."
             )
+            failed = []
+            for username in by_user:
+                # Resolve which file we're targeting
+                user_folder = os.path.join(
+                    examples_dir,
+                    username
+                )
+                task_folder = os.path.join(user_folder, task_info["id"])
+                submission_target = os.path.join(
+                    task_folder,
+                    task_info["target"]
+                )
 
-            # Retrieve expectations list
-            expectations = by_user[username]
-            logging.log(f"Checking expectations for '{username}'...")
-            # Evaluate our rubric against the example submission
-            evaluation = spec.rubric.evaluate(
-                task_info,
-                username,
-                submission_target
-            )
-            # Check the resulting report
-            passed, expl = meta.check_entire_report(evaluation, expectations)
-            # Log the resulting explanation
-            logging.log(expl)
-            status = "passed"
-            if not passed:
-                status = "FAILED"
-                failed.append(username)
-                any_failed = True
-                # Render failed report for inspection
+                # Retrieve expectations list
+                expectations = by_user[username]
+                logging.log(
+                    f"Checking {mode} expectations for '{username}'..."
+                )
+                # Evaluate our rubric against the example submission
+                if mode == "evaluation":
+                    report = spec.rubric.evaluate(
+                        task_info,
+                        username,
+                        submission_target
+                    )
+                else:
+                    tests_target = os.path.join(
+                        task_folder,
+                        task_info.get(
+                            "tests_target",
+                            "test_" + task_info["target"]
+                        )
+                    )
+                    report = spec.rubric.validate(
+                        task_info,
+                        username,
+                        tests_target,
+                        submission_target
+                    )
+
+                # Check the resulting report
+                passed, expl = meta.check_entire_report(report, expectations)
+                # Log the resulting explanation
+                logging.log(expl)
+                status = "passed"
+                if not passed:
+                    status = "FAILED"
+                    failed.append(username)
+                    any_failed = True
+                # Render report for inspection whatever the outcome
                 if os.path.isdir("reports"):
                     cfdir = os.path.join("reports", "__checks__")
                     os.makedirs(cfdir, exist_ok=True)
                     fnbase = os.path.join(
                         cfdir,
-                        f"{task_info['id']}-{username}"
+                        f"{task_info['id']}-{username}-{mode}"
                     )
                     render.render_report(
-                        evaluation,
+                        report,
                         instructions,
                         snippet_markup,
                         fnbase + ".json",
                         fnbase + ".html",
                     )
                     logging.log(f"Wrote report to '{fnbase}.html'")
-            logging.log(f"...done checking '{username}' ({status}).")
-        if len(failed) > 0:
-            logging.log(f"{len(failed)}/{len(by_user)} examples failed.")
-        else:
-            logging.log("All examples met expectations.")
+                else:
+                    logging.log(
+                        "No reports directory, so check report won't be"
+                        " saved."
+                    )
+                logging.log(f"...done checking '{username}' ({status}).")
+            if len(failed) > 0:
+                logging.log(
+                    f"{len(failed)}/{len(by_user)} examples failed."
+                )
+            else:
+                logging.log("All examples met expectations.")
 
-    logging.log("Checking solution code...")
-    soln_file = os.path.join(
-        os.path.dirname(spec.__file__),
-        "soln",
-        task_info["target"]
-    )
-    soln_evaluation = spec.rubric.evaluate(
-        task_info,
-        "__soln__",
-        soln_file
-    )
-    # Check just defaults for solution report
-    passed, expl = meta.check_entire_report(soln_evaluation, [])
-    status = "passed"
-    if not passed:
-        status = "FAILED"
-        any_failed = True
-        # Render failed report for inspection
-        if os.path.isdir("reports"):
-            cfdir = os.path.join("reports", "__checks__")
-            os.makedirs(cfdir, exist_ok=True)
-            fnbase = os.path.join(
-                cfdir,
-                f"{task_info['id']}-__soln__"
+        if by_user is None:
+            logging.log(
+                "Skipping solution check (no expectations for this mode)."
             )
-            render.render_report(
-                soln_evaluation,
-                instructions,
-                snippet_markup,
-                fnbase + ".json",
-                fnbase + ".html",
+        else:
+            logging.log("Checking solution code...")
+            soln_file = os.path.join(
+                os.path.dirname(spec.__file__),
+                "soln",
+                task_info["target"]
             )
-            logging.log(f"Wrote report to '{fnbase}.html'")
-    logging.log(expl)
-    logging.log(f"Check of solution code {status}.")
+            soln_tests_file = os.path.join(
+                os.path.dirname(spec.__file__),
+                "soln",
+                task_info.get(
+                    "tests_target",
+                    "test_" + task_info["target"]
+                )
+            )
+            if mode == "evaluation":
+                soln_report = spec.rubric.evaluate(
+                    task_info,
+                    "__soln__",
+                    soln_file
+                )
+            else:
+                soln_report = spec.rubric.validate(
+                    task_info,
+                    "__soln__",
+                    soln_tests_file,
+                    soln_file
+                )
+            # Check just defaults for solution report
+            passed, expl = meta.check_entire_report(soln_report, [])
+            status = "passed"
+            if not passed:
+                status = "FAILED"
+                any_failed = True
+
+            # Render report for inspection whether or not we failed if
+            # there's a reports directory
+            if os.path.isdir("reports"):
+                cfdir = os.path.join("reports", "__checks__")
+                os.makedirs(cfdir, exist_ok=True)
+                fnbase = os.path.join(
+                    cfdir,
+                    f"{task_info['id']}-__soln__-{mode}"
+                )
+                render.render_report(
+                    soln_report,
+                    instructions,
+                    snippet_markup,
+                    fnbase + ".json",
+                    fnbase + ".html",
+                )
+                logging.log(f"Wrote report to '{fnbase}.html'")
+            else:
+                logging.log(
+                    "No reports directory, so check report won't be saved."
+                )
+
+            logging.log(expl)
+            logging.log(f"Check of solution code {status}.")
 
     logging.log("...done checking expectations.")
 
@@ -475,6 +526,137 @@ def test_specification(task_info, examples_dir):
         return True
 
 
+def validate_tests(
+    task_info,
+    username,
+    user_folder,
+    report_filename,
+    report_html_filename,
+    tests_target=None,
+    target_file=None
+):
+    """
+    Validates a tests file for a single submission, based on task info
+    (specifies the rubric to load), a username (who submitted the task?),
+    and both a tests target and a target file (each either a filename or
+    directory for the tests/submission to be evaluated, or None to determine
+    these automatically from the provided configuration). Note that when
+    the target_file is None, the tests will be run against the solution
+    code for the task, but otherwise, they'll run against whatever
+    file/directory the target file specifies. Creates/overwrites the
+    given report files.
+
+    Writes log messages using the logging sub-module, so redirect those
+    beforehand if you wish.
+    """
+    logging.log(
+        f"Validating tests for task:\n"
+        f"    task: {task_info['id']}\n"
+        f"    user: {username}\n"
+        f"    tests file: {tests_target}\n"
+        f"    target file: {target_file}\n"
+        f"    report: {report_filename}\n"
+        f"    html_report: {report_html_filename}"
+    )
+
+    # Figure out the tests file for this user
+    if tests_target is not None: # explicit
+        submitted_tests = tests_target
+        user_folder = None
+        task_folder = None
+        logging.log(
+            f"Tests are (explicit): {submitted_tests}"
+        )
+    else: # implicit from task/user
+        task_folder = os.path.join(user_folder, task_info["id"])
+
+        # Note the "test_" prefix here
+        submitted_tests = os.path.join(
+            task_folder,
+            "test_" + task_info["target"]
+        )
+
+        logging.log(
+            f"Tests are (implicit): {submitted_tests}"
+        )
+
+    # Figure out the file to test
+    if target_file is not None: # explicit
+        submission_to_test = target_file
+        soln_folder = None
+        logging.log(
+            f"Submission to test is (explicit): {submission_to_test}"
+        )
+    else: # implicit from task/user
+        spec = task_info["specification"]
+        soln_folder = spec.soln_path
+
+        submission_to_test = os.path.join(
+            soln_folder,
+            task_info["target"]
+        )
+
+        logging.log(
+            f"Testing against solution (implicit): {submission_to_test}"
+        )
+
+    # Fatal error if the submitted tests file/directory doesn't exist
+    if not os.path.exists(submitted_tests):
+        logging.log(
+            f"Fatal error: Submitted tests file (or folder)"
+            f" '{submitted_tests}' does not exist"
+        )
+
+        # Log more info on which directories don't exist
+        if user_folder and not os.path.isdir(user_folder):
+            logging.log(f"    No user folder {user_folder}")
+
+        if task_folder and not os.path.isdir(task_folder):
+            logging.log(f"    No task folder {task_folder}")
+
+        exit(1) # Cannot proceed
+
+    # Fatal error if the submission-to-test file/directory doesn't exist
+    if not os.path.exists(submission_to_test):
+        logging.log(
+            f"Fatal error: Submission to test file (or folder)"
+            f" '{submission_to_test}' does not exist"
+        )
+
+        # Log more info on which directories don't exist
+        if soln_folder and not os.path.isdir(soln_folder):
+            logging.log(f"    No solutions folder {soln_folder}")
+
+        exit(1) # Cannot proceed
+
+    # Given tests to run and a submission to run them on, run the tests
+    # and record the results. This produces a report dictionary (see
+    # rubrics.Rubric.validate).
+    logging.log("Running submitted tests...")
+    spec = task_info["specification"]
+    # TODO: This function
+    validation = spec.rubric.validate(
+        task_info,
+        username,
+        submitted_tests,
+        submission_to_test
+    )
+
+    # Now that we've got a tests report, write it to the report file.
+    logging.log(
+        f"Rendering report to '{report_filename}' and"
+        f" '{report_html_filename}'..."
+    )
+    # TODO: This function
+    render.render_tests_report(
+        validation,
+        report_filename,
+        report_html_filename
+    )
+    logging.log("...done validating tests.")
+    return True
+
+
 def evaluate_submission(
     task_info,
     username,
@@ -484,11 +666,11 @@ def evaluate_submission(
     target_file=None
 ):
     """
-    Evaluates a single submission, based on a task info (specifies the
+    Evaluates a single submission, based on task info (specifies the
     rubric to load), a username (who submitted the task?), and a
     target file (either a filename or directory for the submission
     to be evaluated, or None to determine this automatically from the
-    provided configuration). Creates/overwrites the given report file.
+    provided configuration). Creates/overwrites the given report files.
 
     Writes log messages using the logging sub-module, so redirect those
     beforehand if you wish.
@@ -539,9 +721,8 @@ def evaluate_submission(
         exit(1) # Cannot proceed
 
     # Given a submission to evaluate and a Rubric, evaluate that Rubric
-    # in the context of the submission. This produces a report with
-    # 'taskid', 'username', 'evaluation', 'warnings', 'summary', 'table',
-    # and 'context' keys (see rubrics.Rubric.evaluate).
+    # in the context of the submission. This produces a report dictionary
+    # (see rubrics.Rubric.evaluate).
     logging.log("Evaluating rubric...")
     spec = task_info["specification"]
     evaluation = spec.rubric.evaluate(
@@ -666,6 +847,14 @@ def launch_job(
                         if key != "id":
                             record[key] = task[key]
 
+                    # Build a starter URL if tasks.json includes one
+                    if "starter_url" in tasks_data:
+                        record["starter_url"] = (
+                            tasks_data["starter_url"].format(
+                                taskid=taskid
+                            )
+                        )
+
                     # Build a submission URL if tasks.json includes one
                     if "submission_url" in tasks_data:
                         record["submission_url"] = (
@@ -686,7 +875,13 @@ def launch_job(
 
         # Load the task spec. This results in a task-specification
         # module, which should define a 'rubric' variable.
-        contexts.AutoContext.reset(task_info["target"])
+        contexts.AutoContext.reset(
+            task_info["target"],
+            task_info.get(
+                "tests_target",
+                "test_" + task_info["target"]
+            )
+        )
         spec = load.load_task_spec(task_info)
         # Attach the loaded spec back into the task info
         task_info["specification"] = spec
@@ -901,6 +1096,80 @@ def launch_specifications_test(
     )
 
 
+def launch_test_validation(
+    config,
+    taskid,
+    username,
+    log_file=None,
+    tests_target=None,
+    target_file=None,
+    report_filename=None,
+    ignore_cache=False
+):
+    """
+    Validates submitted tests for a task, generating HTML/JSON report
+    files. A configuration object, task ID string, and username string
+    are required. Optional arguments include a log file, a tests target
+    file to validate, a target file to validate tests against, the
+    filename to write our report in, and whether or not to ignore cached
+    reference values.
+    """
+    if username is None:
+        logging.log("Error: A username is required for tests evaluation.")
+        exit(1)
+
+    logging.log(f"Validating {taskid} tests for user {username}...")
+
+    # Figure out user folder
+    user_folder = os.path.join(
+        config.BASE_DIR,
+        config.SUBMISSIONS_DIR,
+        username
+    )
+
+    # Find report directory for this user
+    report_dir = os.path.join(
+        config.BASE_DIR,
+        config.REPORTS_DIR,
+        username
+    )
+
+    # Ensure per-user report directory exists
+    os.makedirs(report_dir, exist_ok=True)
+
+    if report_filename is None:
+        timestamp = time_utils.timestring()
+        report_filename = os.path.join(
+            report_dir,
+            f"{taskid}_validation_{timestamp}.json"
+        )
+        report_html_filename = os.path.join(
+            report_dir,
+            f"{taskid}_validation_{timestamp}.html"
+        )
+    else:
+        report_html_filename = (
+            os.path.splitext(report_filename)[0] + '.html'
+        )
+
+    # Set up and launch
+    launch_job(
+        validate_tests,
+        (
+            username,
+            user_folder,
+            report_filename,
+            report_html_filename,
+            tests_target,
+            target_file,
+        ),
+        config,
+        taskid,
+        log_file,
+        ignore_cache
+    )
+
+
 def launch_evaluation(
     config,
     taskid,
@@ -911,11 +1180,11 @@ def launch_evaluation(
     ignore_cache=False
 ):
     """
-    Evaluates a submitted task, generating a report file. A configuration
-    object, task ID string, and username string are required. Optional
-    arguments include a log file, a target file for evaluation, the
-    filename to write our report in, and whether or not to ignore cached
-    reference values.
+    Evaluates a submitted task, generating HTML/JSON report files. A
+    configuration object, task ID string, and username string are
+    required. Optional arguments include a log file, a target file for
+    evaluation, the filename to write our report in, and whether or not
+    to ignore cached reference values.
     """
     if username is None:
         logging.log("Error: A username is required for task evaluation.")
