@@ -52,12 +52,14 @@ def load_task_spec(task_info):
     - starter_src: the source code for the main starter file
         (or an empty string if there is no starter file or if the
         task requires more than one file)
-    - soln_files: all files in the solution directory (not full paths)
-    - starter_files: all files in the starter directory (not full paths)
-    - helper_files: a list of strings naming files which are in the
-        starter directory and the solution directory but which aren't
-        the main task file (or directory) itself. These are just the
-        file names, not the full paths.
+    - soln_files: all files/directories in the solution directory (not
+        full paths)
+    - starter_files: all files/directories in the starter directory (not
+        full paths)
+    - helper_files: a list of strings naming files/directories which are
+        in the starter directory and the solution directory but which
+        aren't the main task file (or directory) itself. These are just
+        the file names, not the full paths.
     """
     # Set up sys.path and import specifically:
     # Note: Relevant directories will need __init__.py files!
