@@ -116,6 +116,15 @@ A `potluck.mast` pattern for a triple for loop comprehension with a filter.
 """
 
 
+SINGLE_LOOP_COMPREHENSION_PATTERNS = [
+    SIMPLE_COMPREHENSION_PATTERN,
+    FILTER_COMPREHENSION_PATTERN
+]
+"""
+`potluck.mast patterns for just single list comprehensions with and
+without filtering.
+"""
+
 ALL_REASONABLE_COMPREHENSION_PATTERNS = [
     SIMPLE_COMPREHENSION_PATTERN,
     FILTER_COMPREHENSION_PATTERN,
@@ -133,12 +142,36 @@ Note: does not include dictionary comprehensions (see below)
 We will simply hope that students do not use quadruple comprehensions...
 """
 
+ALL_SINGLE_LOOP_GENERATOR_EXPRESSION_PATTERNS = (
+    SINGLE_LOOP_COMPREHENSION_PATTERNS
+  + [
+      p.replace('[', '(').replace(']', ')')
+      for p in SINGLE_LOOP_COMPREHENSION_PATTERNS
+    ] # generator expressions
+  + [
+      p.replace('[', '{').replace(']', '}')
+      for p in SINGLE_LOOP_COMPREHENSION_PATTERNS
+    ] # set comprehensions
+  + [
+      p.replace('[ _', '{ _: _').replace(']', '}')
+      for p in SINGLE_LOOP_COMPREHENSION_PATTERNS
+    ] # dictionary comprehensions
+)
+"""
+`potluck.mast` patterns for single-loop comprehensions including set and
+dictionary comprehensions and generator expressions.
+"""
+
 ALL_GENERATOR_EXPRESSION_PATTERNS = (
     ALL_REASONABLE_COMPREHENSION_PATTERNS
   + [
       p.replace('[', '(').replace(']', ')')
       for p in ALL_REASONABLE_COMPREHENSION_PATTERNS
     ] # generator expressions
+  + [
+      p.replace('[', '{').replace(']', '}')
+      for p in ALL_REASONABLE_COMPREHENSION_PATTERNS
+    ] # set comprehensions
   + [
       p.replace('[ _', '{ _: _').replace(']', '}')
       for p in ALL_REASONABLE_COMPREHENSION_PATTERNS
@@ -146,7 +179,7 @@ ALL_GENERATOR_EXPRESSION_PATTERNS = (
 )
 """
 Patterns for up to triple-loop list comprehensions, plus equivalent
-generator expressions and dictionary comprehensions.
+generator expressions and set- and dictionary-comprehensions.
 """
 
 
@@ -164,14 +197,13 @@ ALL_LOOP_AND_COMPREHENSION_PATTERNS = (
 although highly-nested comprehensions are not included.
 """
 ALL_SINGLE_LOOP_AND_COMPREHENSION_PATTERNS = (
-    ALL_FOR_AND_WHILE_LOOP_PATTERNS + [
-        SIMPLE_COMPREHENSION_PATTERN,
-        FILTER_COMPREHENSION_PATTERN
-    ]
+    ALL_FOR_AND_WHILE_LOOP_PATTERNS
+  + ALL_SINGLE_LOOP_GENERATOR_EXPRESSION_PATTERNS
 )
 """
 `potluck.mast` patterns for all loop/comprehension patterns that are
-single loops.
+single loops, including generator expressions, set comprehensions, and
+dictionary comprehensions.
 """
 
 
