@@ -728,6 +728,17 @@ class Snippet(specifications.TestGroup):
         self.glamers.append(append_audio_result)
         return self
 
+    def show_memory_report(self):
+        """
+        Adds a glamer that augments output by appending a "Memory Report"
+        prompt followed by a memory report showing the structure of the
+        "ref_value" context slot.
+
+        Returns self for chaining.
+        """
+        self.glamers.append(append_memory_report)
+        return self
+
     def show_file_contents(self):
         """
         Adds a glamer that augments output by appending a "File"
@@ -1381,6 +1392,27 @@ def append_audio_result(markup, context):
     )
 
 
+def append_memory_report(markup, context):
+    """
+    Given some HTML markup and a context dictionary, creates a memory
+    report showing the structure of the object in the "ref_value" context
+    slot, and appends that to the markup, using a 'Memory Report' tag.
+    """
+    # Do nothing if no ref_value is available
+    if "ref_value" not in context:
+        return markup
+
+    obj = extract(context, "ref_value")
+
+    report = html_tools.escape(specifications.memory_report(obj))
+    formatted = "<pre>{report}</pre>".format(report=report)
+    return (
+        markup
+      + '\n<span class="prompt special">Memory\nReport</span>'
+      + formatted
+    )
+
+
 def append_file_contents(markup, context):
     """
     Given some HTML markup and a context dictionary, turns the
