@@ -29,6 +29,21 @@ def grep(filename, fragment, writeTo):
                     fout.write(line)
 
 
+def readNums(filename):
+    """
+    Given a filename, reads a list of floating-point numbers from the
+    file, one per line, skipping lines that cannot be interpreted as
+    numbers.
+    """
+    result = []
+    with open(filename, 'r') as fin:
+        for line in fin:
+            # ERROR: no try/except
+            result.append(float(line.strip()))
+
+    return result
+
+
 def addRecords(filename, records):
     """
     Given a filename and a list of strings, adds one line to the file
