@@ -19,18 +19,5 @@ def arpeggio(startPitch, noteDuration):
     addNote(0.5)
     halfStepUp(3)
     addNote(0.5)
-    halfStepDown(6)
+    halfStepDown(6) # wrong on way down
     addNote(0.5)
-
-
-# Only one test
-import optimism as opt
-opt.captureOutput()
-
-arpeggio(C4, 0.5)
-opt.testCase(printTrack())
-
-opt.expectOutputContains("a 0.5s keyboard note at C4") # first
-opt.expectOutputContains("and a 0.5s keyboard note at C4") # last
-opt.expectOutputContains("note at E4") # 2 OR 4
-opt.expectOutputContains("note at G4") # 3
