@@ -30,7 +30,7 @@ def whatever(a, b):
 import optimism as opt
 
 m = opt.testFunction(whatever)
-m.case(True, True).checkResult(1)
-m.case(True, False).checkResult(2)
-m.case(False, True).checkResult(2)
-m.case(False, False).checkResult(3)
+m.case(True, True).checkReturnValue(1)
+m.case(True, False).checkReturnValue(2)
+m.case(False, True).checkReturnValue(2)
+m.case(False, False).checkReturnValue(3)
