@@ -30,6 +30,23 @@ def grep(filename, fragment, writeTo):
                     fout.write(line)
 
 
+def readNums(filename):
+    """
+    Given a filename, reads a list of floating-point numbers from the
+    file, one per line, skipping lines that cannot be interpreted as
+    numbers.
+    """
+    result = []
+    with open(filename, 'r') as fin:
+        for line in fin:
+            try:
+                result.append(float(line.strip()))
+            except Exception:
+                pass
+
+    return result
+
+
 def addRecords(filename, records):
     """
     Given a filename and a list of strings, adds one line to the file
