@@ -40,8 +40,8 @@ meta.example("perfect")
 
 meta.example("imperfect")
 
-meta.expect("failed", "procedure", "core", "import the turtleBeads module")
-meta.expect("failed", "procedure", "core", "call left")
-meta.expect("failed", "procedure", "extra", "call drawCircle")
-meta.expect("failed", "procedure", "extra", "call begin_fill")
-meta.expect("failed", "procedure", "extra", "call end_fill")
+meta.expect("failed", "import the turtleBeads module")
+meta.expect("failed", "call left")
+meta.expect("failed", "call drawCircle")
+meta.expect("failed", "call begin_fill")
+meta.expect("failed", "call end_fill")
