@@ -28,7 +28,7 @@ the kernel if you're in a notebook, or stop and restart Python elsehow).
 """
 
 import urllib, urllib.parse, urllib.request, urllib.error
-import json, sys
+import json, sys, datetime
 
 try:
     import optimism
@@ -40,7 +40,7 @@ except ModuleNotFoundError:
     )
     optimism = None
 
-__version__ = 1.0
+__version__ = 1.2
 """
 Version number for this script, separate from the `potluck` and
 `potluck_server` versions just in case it's useful.
@@ -48,8 +48,17 @@ Version number for this script, separate from the `potluck` and
 
 AUTHORS = None
 """
-A list of strings indicating which user account(s) submissions should be
-assigned to. Use `setAuthors` and/or `collectAuthors` to update this.
+A pair including list of strings indicating which user account(s)
+submissions should be assigned to, followed by a `datetime.datetime`
+object indicating when that information was provided. Use `setAuthors`
+and/or `collectAuthors` to update this; use `getAuthors` to retrieve
+authors in a way that respects the timeout.
+"""
+
+AUTHORS_TIMEOUT = datetime.timedelta(hours=3)
+"""
+The duration after which we'll ask for authors again during a single
+session. Author information is not cached between sessions.
 """
 
 BASE_URL = "http://www.example.com/potluck/"
@@ -115,29 +124,6 @@ def setup(url=None, course=None, semester=None):
         SEMESTER = semester
 
 
-def establishAuthors():
-    """
-    Checks that author info is available in the `AUTHORS` variable and
-    calls `collectAuthors` if not. Prints a message and returns False if
-    after prompting no author info is available; returns True otherwise.
-    """
-    global AUTHORS
-
-    if AUTHORS is None or len(AUTHORS) == 0:
-        print("You haven't yet specified authors for this session.")
-        collectAuthors()
-
-    # Might still be None if user didn't enter any names
-    if AUTHORS is None or len(AUTHORS) == 0:
-        print(
-            "No author names provided; aborting submission.",
-            file=sys.stderr
-        )
-        return False
-
-    return True
-
-
 def deliver():
     """
     Collects outcomes registered with the current `optimism` test suite
@@ -160,7 +146,12 @@ def deliver():
     """
     global optimism
 
-    if not establishAuthors():
+    authors = getAuthors()
+    if authors is None:
+        print(
+            "Unable to submit without authorship information. Please"
+            " try again."
+        )
         return
 
     if optimism is None:
@@ -195,7 +186,7 @@ def deliver():
     # Get outcomes from optimism
     outcomes = optimism.listOutcomesInSuite(suiteName)
 
-    doDelivery(AUTHORS, suiteName, outcomes, codeBlocks)
+    doDelivery(authors, suiteName, outcomes, codeBlocks)
 
 
 class NoCondition:
@@ -230,15 +221,17 @@ def deliverOutcome(
     automatically, so that a single call to this function can suffice
     for submission without needing to call other stuff beforehand:
 
-    - 'suite' will be used in place of the current optimism test
+    - `suite` will be used in place of the current optimism test
         suite name, with any prefix established by `prefix` still
         applied. In cases where `suite` is not set and `optimism` is not
         available, the string "default" will be used.
-    - 'grabCode' should be a list of strings; code marked using
+    - `grabCode` should be a list of strings; code marked using
         `optimism.mark` and those strings will be included in the
         submission. If optimism is not available, setting `grabCode` to
         anything other than `None` or an empty list will result in an
-        error.
+        error. Even when `grabCode` is set to None, if optimism is
+        available and there's a marked code segment that shares the
+        suite name provided, it will be included.
 
     The `note` and `failNote` arguments will be used as the outcome
     message value for the constructed outcome. Note that there is no way
@@ -251,7 +244,12 @@ def deliverOutcome(
     """
     global optimism
 
-    if not establishAuthors():
+    authors = getAuthors()
+    if authors is None:
+        print(
+            "Unable to submit without authorship information. Please"
+            " try again."
+        )
         return
 
     if optimism is None:
@@ -277,11 +275,12 @@ def deliverOutcome(
             )
         for mark in grabCode:
             codeBlocks.append(
-                [
-                    "code for '{}'".format(mark),
-                    optimism.getMarkedCode(mark)
-                ]
+                [f"code marked '{mark}'", optimism.getMarkedCode(mark)]
             )
+    elif optimism is not None:
+        suiteBlock = optimism.getMarkedCode(suite)
+        if suiteBlock is not None:
+            codeBlocks.append([f"code marked '{suite}'", suiteBlock])
 
     # Construct our outcome
     outcomes = [
@@ -292,7 +291,7 @@ def deliverOutcome(
         ]
     ]
 
-    doDelivery(AUTHORS, suite, outcomes, codeBlocks)
+    doDelivery(authors, suite, outcomes, codeBlocks)
 
 
 def doDelivery(authors, suiteName, outcomes, codeBlocks):
@@ -335,7 +334,7 @@ def doDelivery(authors, suiteName, outcomes, codeBlocks):
         }
     )
     payload = bytes(encoded, encoding="utf-8")
-    print(f"Submitting to {destination}...")
+    print(f"Submitting to {destination} ...")
     try:
         with urllib.request.urlopen(destination, payload) as response:
             result = response.read().decode("utf-8")
@@ -350,7 +349,17 @@ def doDelivery(authors, suiteName, outcomes, codeBlocks):
         return
 
     if result.startswith("Submission accepted:"):
-        print(result)
+        if "this submission is NOT complete" in result:
+            print(result)
+            print(
+                (
+                    "Check the output above for any error messages, and"
+                    " double-check your answer."
+                ),
+                file=sys.stderr
+            )
+        else:
+            print(result)
     else:
         print(
             (
@@ -419,7 +428,7 @@ def setAuthors(*usernames):
     global AUTHORS
     if not all(isinstance(un, str) for un in usernames):
         raise TypeError("Each username must be a string.")
-    AUTHORS = usernames
+    AUTHORS = (usernames, datetime.datetime.now())
 
 
 def collectAuthors():
@@ -443,3 +452,42 @@ def collectAuthors():
         setAuthors(*allBits)
     else:
         print("You didn't enter any author names.")
+
+
+def getAuthors():
+    """
+    Gets the current authors list, asking the user to enter that
+    information if it's not set up yet, and/or if the existing
+    information has expired.
+
+    Returns `None` if the user enters invalid info.
+    """
+    global AUTHORS
+    now = datetime.datetime.now()
+    if AUTHORS is None or len(AUTHORS[0]) == 0:
+        print(
+            "You have not yet entered authorship information for this"
+            " session."
+        )
+        collectAuthors()
+
+    if AUTHORS is None or len(AUTHORS[0]) == 0:
+        return None
+
+    if AUTHORS[1] + AUTHORS_TIMEOUT < now:
+        print(
+            "The stored authors information has expired, please"
+            " re-enter the authors list based on who you are working"
+            " with now."
+        )
+        collectAuthors()
+
+    if (
+        AUTHORS is None
+     or len(AUTHORS[0]) == 0
+     or AUTHORS[1] + AUTHORS_TIMEOUT < now
+    ):
+        AUTHORS = None
+        return None
+
+    return AUTHORS[0]
