@@ -8,7 +8,7 @@ storage.py
 # Attempts at 2/3 dual compatibility:
 from __future__ import print_function
 
-__version__ = "0.3.1"
+__version__ = "0.3.2"
 
 import sys, os, shutil, subprocess, threading, copy
 import time, datetime
@@ -545,7 +545,7 @@ def get_admin_info(course, semester):
     try:
         result = load_or_get_cached(
             filename,
-            assume_fresh=_CONFIG.get("assume_fresh", 1)
+            assume_fresh=_CONFIG.get("ASSUME_FRESH", 1)
         )
     except Exception:
         flask.flash("Failed to read admin info file '{}'!".format(filename))
@@ -1283,11 +1283,47 @@ def set_evaluation(
         taskid
     )
 
+    # Get task info for this course/semester so we can access per-course
+    # config values...
+    task_info = get_task_info(course, semester)
+    relevant_task = task_info.get("tasks", {}).get(taskid, {})
+    relevant_projects = [
+        p
+        for p in task_info.get("projects", task_info.get("psets", {}))
+        if p["id"] == prid
+    ]
+    if len(relevant_projects) > 0:
+        relevant_project = relevant_projects[0]
+    else:
+        relevant_project = {}
+
     # Generate a timestamp for the info
     timestring = potluck.time_utils.timestring()
 
+    # Get SCORE_BASIS and TIMELINESS_POINTS values from
+    # task/project/task_info/config
+    score_basis = relevant_task.get(
+        "SCORE_BASIS",
+        relevant_project.get(
+            "SCORE_BASIS",
+            task_info.get(
+                "SCORE_BASIS",
+                _CONFIG.get("SCORE_BASIS", 100)
+            )
+        )
+    )
+    timeliness_basis = relevant_task.get(
+        "TIMELINESS_POINTS",
+        relevant_project.get(
+            "TIMELINESS_POINTS",
+            task_info.get(
+                "TIMELINESS_POINTS",
+                _CONFIG.get("TIMELINESS_POINTS", 10)
+            )
+        )
+    )
+
     # Convert to a number if we can
-    score_basis = _CONFIG.get("SCORE_BASIS", 100)
     if override != "":
         try:
             override = float(override)
@@ -1311,7 +1347,6 @@ def set_evaluation(
             )
 
     # Convert to a number if we can
-    timeliness_basis = _CONFIG.get("TIMELINESS_POINTS", 10)
     if timeliness != "":
         try:
             timeliness = float(timeliness)
@@ -2080,6 +2115,7 @@ class AsStudentInfo(View):
         result = {}
         for row in reader:
             entry = {}
+            # TODO: Get this remap on a per-course basis!!!
             for key in _CONFIG["REMAP_STUDENT_INFO"]:
                 entry[_CONFIG["REMAP_STUDENT_INFO"][key]] = row.get(key)
             entry["username"] = entry["email"].split('@')[0]
